local sr = KSR

local function startswith(self, str)
	return self:sub(1, #str) == str
end


function data_urlencode()

	local str = sr.pv.get("$var(push_caller_numb)");
	local number = url_encode(str);
	sr.pv.sets("$var(push_caller_numb)", number)

	local str = sr.pv.get("$var(push_calling_numb)");
	local number = url_encode(str);
	sr.pv.sets("$var(push_calling_numb)", number)

	local str = sr.pv.get("$var(push_caller_name)");
	local name = url_encode(str);
	sr.pv.sets("$var(push_caller_name)", name)

	local str = sr.pv.get("$var(push_calling_name)");
	local name = url_encode(str);
	sr.pv.sets("$var(push_calling_name)", name)

	local str = sr.pv.get("$var(push_message)");
	local name = url_encode(str);
	sr.pv.sets("$var(push_message)", name)

end


function url_encode(str)
	if (str) then
		str = string.gsub (str, "\n", "\r\n")
		str = string.gsub (str, "([^%w %-%_%.%~])",
		function (c) return string.format ("%%%02X", string.byte(c)) end)
		str = string.gsub (str, " ", "+")
	end
	return str
end


function add_ice_candidates_to_wms_network()
	local math = require"math"
	math.randomseed(os.time())

	local function gen_random_string()
		local chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
		local length = 16
		local randomString = ''

		local charTable = {}
		for c in chars:gmatch"." do
			table.insert(charTable, c)
		end

		for i = 1, length do
			randomString = randomString .. charTable[math.random(1, #charTable)]
		end

		return randomString
	end

	local sdp = sr.pv.gete("$avp(write_sdp_out)")
	local ips = sr.pv.gete("$sht(globals=>pbx_own_ips)")
	local dst_pbx_local_addrs = sr.pv.gete("$sht(wms_neighbors_localaddrs=>$hdr(X-To-WMSNet))") or ""
	local ip_table = {}
	local added_ips = {}

	if type(ips) ~= "string" then
		return
	end

	for ip in string.gmatch(ips, '([^,]+)') do
		if not string.find(dst_pbx_local_addrs, ip) then
			ip = ip:gsub("\"", "")
			if not added_ips[ip] then
				added_ips[ip] = true
				--[[
				ICE candidate priority https://datatracker.ietf.org/doc/html/rfc5245#section-4.1.2.1
				priority = (2^24)*(type preference) +
							(2^8)*(local preference) +
							(2^0)*(256 - component ID)
				--]]
				table.insert(ip_table, {ip=ip, foundation=gen_random_string(), priority=(2113929216+256*#ip_table)})
			else
				sr.log("notice", string.format("Skip adding ip [%s] to candidates list due to we already added this one to the list of candidates", ip))
			end
		else
			sr.log("notice", string.format("Skip adding ip [%s] to candidates list due to the existence of the same ip at the destination pbx", ip))
		end
	end

	local res = ""
	for line in string.gmatch(sdp, "(.-)\r?\n") do
		if startswith(line, "a=candidate") then
			if line:find("0.0.0.0") then
				for _, v in pairs(ip_table) do
					local cand = line:gsub("a=candidate:(%w+) (%d) (%w+) (%d+) 0%.0%.0%.0", "a=candidate:"..v["foundation"].." %2 %3 "..v["priority"].." "..v["ip"])
					res = res..cand.."\r\n"
				end
			else
				res = res..line.."\r\n"
			end
		else
			res = res..line.."\r\n"
		end
	end
	if res ~= "" then
		sr.pv.sets("$var(sdp_wms_network)", res)
	end
end


function check_permissions(src_gr, dst_gr, what)
	sr.pv.sets("$var(check_permissions_result)", "")
	local license = sr.pv.get("$avp(licenseType)")
	local rv = require"common.permissions":check(src_gr, "yesno", what, dst_gr, license)
	sr.pv.sets("$var(check_permissions_result)", rv)
end
